"""
DaaS Service client base.
"""
from urllib.parse import urljoin

from .base_client import BaseClient


class ServiceBaseClient:
    """
    Abstract base service client class for connecting with Daas Platform.
    Not intended to be used directly, rather one of its subclasses.
    """

    def __init__(
        self,
        client: BaseClient,
        service_name: str,
        service_version: str = "1.0",
        is_dev: bool = False,
    ) -> None:
        """
        Setup the service base client.

        :param client: client to use for connection.
        :param service_name: service name.
        :service version: service version, default is 1.0. NB. mabye should use 'latest'
        :is_dev: true for dev version
        """
        if not client or not isinstance(client, BaseClient):
            raise ValueError("Client should be an BaseClient instance.")

        self._client = client
        self._service_name = service_name
        self._service_version = service_version
        self._is_dev = is_dev

    @property
    def service_name(self) -> str:
        """
        Get name of the service.

        By default, used for constructing the url unless api_url is overwridden.
        """
        return self._service_name

    @property
    def service_version(self) -> str:
        """
        Version of the api being used
        """
        return self._service_version

    def api_url(self, extra_path: str = None) -> str:
        """
        Get a url for the api including any specified extra path

        :param extra_path: extra path to add to the base url. Defaults to None.
        """
        if self._is_dev:
            url = urljoin(
                self._client.server_url,
                f"/api/{self.service_name}-dev/{self.service_version}/",
            )
        else:
            url = urljoin(
                self._client.server_url,
                f"/api/{self.service_name}/{self.service_version}/",
            )

        if extra_path:
            url = urljoin(url, extra_path)

        return url



"""
Base client to daas service apis
"""

import logging
import requests

from typing import Union, Dict, List
from requests import Response, HTTPError

logger = logging.getLogger(__name__)


class BaseClient:
    """
    Class for connecting with APIs.
    """

    def __init__(
        self,
        server_url: str,
        token: str,
    ):
        """
        Setup the new client
        :param server_url: url of the server without any path e.g. https://www.test.com
        :param token: credentials used for connection
        """
        self._server_url = server_url
        self._token = token

    @property
    def server_url(self) -> str:
        """Url of the API server"""
        return self._server_url

    @property
    def credentials(self) -> str:
        """Credentials used for connection"""
        return self._token

    def get_headers(self):
        """
        Get needed http headers, including authorization bearer token.
        Raises:
            NotImplementedError: Should be implemented by subclasses.
        """
        return {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self._token}",
        }

    def get(self, url: str, ok_status_codes: List = None) -> Response:
        """GET from the specified url
        :param url: url to GET from to
        :param ok_status_codes: list of status code indicating success
        Raises:
            HTTPError: Raised if ok_status_codes are passed and the get returns a different status
        """
        headers = self.get_headers()
        response = requests.get(url, headers=headers)
        if ok_status_codes is not None and response.status_code not in ok_status_codes:
            raise HTTPError(response=response)
        return response

    def get_returning_json(self, url: str, ok_status_codes: list = None) -> Dict:
        """
        Get data from the specified url in json format.
        To be able to do a conversion to json we typically need a valid http response so
        pass this in ok_status_codes.
        :param url: url to GET from to
        :param ok_status_codes: list of status code indicating success. Defaults to [200].
        Raises:
            HTTPError: Raised if the get returns a status other than those in ok_status_codes
        """
        if ok_status_codes is None:
            ok_status_codes = [200]
        response = self.get(url, ok_status_codes)
        return response.json()

    def post(
        self, url: str, data: Union[str, Dict], ok_status_codes: List = None
    ) -> Response:
        """
        POST data to the specified url

        :param url: url to POST to
        :param data: json data as string or dict to send as the body
        :param ok_status_codes: list of status code indicating success

        Raises:
            HTTPError: Raised if ok_status_codes are passed and the post returns a different status
        """
        headers = self.get_headers()

        # determine whether to send to requests as data or json
        _json = None
        if isinstance(data, Dict):
            _json = data
            data = None

        response = requests.post(url, data=data, json=_json, headers=headers)
        if ok_status_codes is not None and response.status_code not in ok_status_codes:
            raise HTTPError(response=response)
        return response

    def post_returning_json(
        self, url: str, data: Union[str, Dict], ok_status_codes: list = None
    ) -> Dict:
        """
        Post data to the specified url and get the result in json format.

        To be able to do a conversion to json we typically need a valid http response so
        pass this in ok_status_codes.

        :param url: url to POST to
        :param data: json data as string or dict to send as the body
        :param ok_status_codes: Status codes indicating successful call. Defaults to [200].

        Raises:
            HTTPError: Raised if the get returns a status other than those in ok_status_codes
        """
        if ok_status_codes is None:
            ok_status_codes = [200]
        response = self.post(url, data, ok_status_codes)
        return response.json()



"""
Entitlements client for working with Daas entitlments API.
"""
from typing import Dict, List

from ..base_client import BaseClient
from ..service_base_client import ServiceBaseClient


class EntitlementsClient(ServiceBaseClient):
    """
    Client for working with Daas entitlements APIs.
    """

    def __init__(
        self, client: BaseClient, service_version: str = "v1", is_dev: bool = False
    ) -> None:
        """
        Set up the EntitlementsClient

        :param client: client to use for connection
        :param service_version (optional): service version defaults to '1.0'.
        """
        super().__init__(client, "entitlements", service_version, is_dev)

    def is_healthy(self) -> bool:
        """
        Returns health status of the API
        """
        response = self._client.get(self.api_url("health-check/"))
        return response.status_code == 200

    def list_groups(self) -> List[Dict]:
        """
        List groups
        """
        response_json = self._client.get_returning_json(self.api_url("groups/"))
        return response_json

    def list_group_members(self, group: str) -> dict:
        """
        List members in a group
        :param group: The id of the group.
        """
        return self._client.get_returning_json(self.api_url(f"groups/{group}/members"))

    def get_product(self, product_id):
        """
        Get info for a product.
        :param product_id: The id of the group.
        """
        return self._client.get_returning_json(self.api_url(f"products/{product_id}"))
